@@ -66,7 +66,7 @@
  */
 - (NJBTransactionResult *)downloadFileList
 {
-	NSLog(@"Getting file");
+	NSLog(@"Getting file list");
 	[fullItemArray release];
 	fullItemArray = [theNJB dataFiles];
 	
@@ -162,20 +162,14 @@
 	NJBQueueItem *copyFile = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(uploadFile:) withObject:dataFile];
 	[copyFile setStatus:STATUS_UPLOADING_FILE];
 	
-	NJBQueueItem *addToFileList = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(addObject:) withObject:dataFile withRunInMainThread:YES];
-	[addToFileList setDisplayStatus:NO];
-	
-	// call searchNJBTable to pretend the search list changed to update
-	NJBQueueItem *updateFileList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
-	[updateFileList setRunInMainThread:YES];
-	[updateFileList setDisplayStatus:NO];
+	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToFullItemArray:) withObject:dataFile withRunInMainThread:YES];
+	[addToList setDisplayStatus:NO];
 	
 	NSString *description = [NSString stringWithFormat:@"Uploading file '%@'", [dataFile filename]];
-	[self addToQueue:copyFile withSubItems:[NSArray arrayWithObjects:addToFileList, updateFileList, nil] withDescription:description];
+	[self addToQueue:copyFile withSubItems:[NSArray arrayWithObject:addToList] withDescription:description];
 	
 	[copyFile release];
-	[addToFileList release];
-	[updateFileList release];
+	[addToList release];
 }
 
 /* delete the item from the NJB
