@@ -9,10 +9,13 @@
  * to read/write id3 tags
  */
 
+#include "wmaread.h"
 #import "ID3Tagger.h"
 #include <id3/tag.h>
 #include <id3/misc_support.h>
 #import "MP3Header.h"
+#import "WMATagger.h"
+#import "UnicodeWrapper.h"
 
 @interface ID3Tagger (PrivateAPI)
 // NB these can't be delcared in ID3Tagger.h as would then include id3/tag.h in other files which causes errors
@@ -87,10 +90,16 @@
 
 /* read the id3 tag and return it as
  * a track object
- * will get the length of wav files but not wma
+ * will pass on to WMATagger if a wma file
  */
 - (Track *)readTrack
 {	
+	// if is WMA file, we can't do anything here, pass on to WMATagger
+	if ([self codec] == CODEC_WMA)
+	{
+		return [WMATagger readTrack:filename];
+	}
+	
 	// check file exists
 	NSFileManager *fileManager = [[NSFileManager alloc] init];
 	// this is used below to get filesize
@@ -102,11 +111,20 @@
 		
 	[track setFilesize:[[fileAttributes objectForKey:NSFileSize] intValue]];
 	[track setCodec:[self codec]];
+	[track setFullPath:filename];
 	
-	if ([self codec] == CODEC_MP3)
+  if ([self codec] == CODEC_MP3)
 	{
 		ID3_Tag myTag;
-		myTag.Link([filename UTF8String]);
+		// ID3V2 is the default - I can't get it to load unicode if not
+		size_t id3Size = myTag.Link([filename fileSystemRepresentation], ID3TT_ID3V2);
+		if (id3Size == 0)
+		{
+			// try anything else
+			NSLog(@"no ID3V2 tag found, trying something else");
+			myTag.Clear();
+			myTag.Link([filename fileSystemRepresentation]);
+		}
 		
 		[track setTitle:[self frameContentsFromTag:&myTag frameID:ID3FID_TITLE]];
 		[track setAlbum:[self frameContentsFromTag:&myTag frameID:ID3FID_ALBUM]];
@@ -120,6 +138,7 @@
 			if ([genre characterAtIndex:0] == 40)
 			{
 				NSRange bracketPos = [genre rangeOfCharacterFromSet:[NSCharacterSet characterSetWithCharactersInString:@")"]];
+				
 				if (bracketPos.length == 1 && bracketPos.location == [genre length] - 1)
 				{
 					NSRange numberRange;
@@ -135,7 +154,7 @@
 			}
 		}
 		[track setGenre:genre];
-
+		
 		// this may be of the form a/b for track a out of b but parsing to integer takes care of this
 		[track setTrackNumber:[self intFrameContentsFromTag:&myTag frameID:ID3FID_TRACKNUM]];
 		[track setYear:[self intFrameContentsFromTag:&myTag frameID:ID3FID_YEAR]];
@@ -150,19 +169,14 @@
 		[track setLength:header->time];*/
 		
 		myTag.Clear();
-		
-		[track setFullPath:filename];
+	
 	}
-	else
+	else if ([self codec] == CODEC_WAV)
 	{
 		// use filename as title, but strip extension
 		NSString *title = [[filename lastPathComponent] stringByDeletingPathExtension];
 		[track setTitle:title];
-		// todo: can we get the length of WMA files?
-		if ([self codec] == CODEC_WAV)
-		{
-			[track setLength:[self wavFileLength:filename]];
-		}
+		[track setLength:[self wavFileLength:filename]];
 	}
 	
 	[fileManager release];
@@ -173,18 +187,47 @@
  */
 - (NSString *)frameContentsFromTag:(ID3_Tag *)tag frameID:(ID3_FrameID)frameID
 {
-	NSString *ret = nil;
-	ID3_Frame *frame = tag->Find(frameID);
+	ID3_Frame *frame = tag->Find(frameID, ID3FN_TEXTENC, (luint) ID3TE_UNICODE);
 	if (frame != NULL)
 	{
-		char *str = ID3_GetString(frame, ID3FN_TEXT);
-		if (str != NULL)
+		ID3_Field *field = frame->GetField(ID3FN_TEXT);
+		if (field != NULL)
 		{
-			ret = [NSString stringWithCString:str];
-			delete [] str;
+			field->SetEncoding(ID3TE_UTF16);
+			// docs say len is no. of characters - I think it's number of bytes
+			size_t len = field->Size()/2;
+			unicode_t *uni = new unicode_t[len + 1];
+			field->Get(uni, len + 1);
+			// check is terminated
+			uni[len] = 0;
+			NSString *uniString = [UnicodeWrapper stringFromUTF16:uni];
+			delete [] uni;
+			return uniString;
 		}
 	}
-	return ret;
+	else
+	{	
+		frame = tag->Find(frameID);
+		if (frame != NULL)
+		{
+			ID3_Field *field = frame->GetField(ID3FN_TEXT);
+			if (field != NULL)
+			{
+				const char *rawText = field->GetRawText();
+				NSData *data = nil;
+				if (field->Size() == 0)
+					return [NSString stringWithString:@""];
+				if (rawText[field->Size() - 1] == 0)
+					// ignore the NULL terminator
+					data = [NSData dataWithBytes:field->GetRawText() length:(field->Size() - 1)];
+				else
+					data = [NSData dataWithBytes:field->GetRawText() length:field->Size()];
+				NSString *asciiString = [[NSString alloc] initWithData:data encoding:NSISOLatin1StringEncoding];
+				return [asciiString autorelease];
+			}
+		}
+	}
+	return nil;
 }
 
 /* returns an integer value from the frame
@@ -201,11 +244,12 @@
 	ID3_Frame *frame = new ID3_Frame(frameID);
 	if (frame)
 	{
-		ID3_Field *field = frame->GetField(ID3FN_TEXT);
-		if (field != NULL)
-		{
-			field->Set([contents cString]);
-		}
+		frame->GetField(ID3FN_TEXT)->SetEncoding(ID3TE_UNICODE);
+		unicode_t *uni = [UnicodeWrapper UTF16FromString:contents];
+		frame->GetField(ID3FN_TEXT)->Set(uni);
+		free(uni);
+		// is this necessary?
+		frame->GetField(ID3FN_TEXTENC)->Set(ID3TE_UTF16);
 		tag->AttachFrame(frame);
 	}
 }
@@ -216,7 +260,7 @@
 - (void)writeTrack:(Track *)track
 {
 	ID3_Tag myTag;
-	myTag.Link([[track fullPath] UTF8String]);
+	myTag.Link([[track fullPath] fileSystemRepresentation], ID3TT_ALL);
 	
 	ID3_RemoveTitles(&myTag);
 	[self setFrameContentsForTag:&myTag frameID:ID3FID_TITLE contents:[track title]];
@@ -226,7 +270,7 @@
 	[self setFrameContentsForTag:&myTag frameID:ID3FID_LEADARTIST contents:[track artist]];
 	ID3_RemoveGenres(&myTag);
 	[self setFrameContentsForTag:&myTag frameID:ID3FID_CONTENTTYPE contents:[track genre]];
-		
+	
 	int nTotal = 0;
 	ID3_Frame *frame = myTag.Find(ID3FID_TRACKNUM);
 	if (frame != NULL)
@@ -247,6 +291,7 @@
 		}
 	}
 
+	ID3_RemoveTracks(&myTag);
 	ID3_AddTrack(&myTag, [track trackNumber], nTotal, true);
 	
 	ID3_RemoveYears(&myTag);
