@@ -16,11 +16,13 @@ typedef enum _activeObjects {OBJECT_NJB_TABLE = 0, OBJECT_FILE_BROWSER = 1, OBJE
 	IBOutlet NSBrowser *fileBrowser;
 	IBOutlet NSButton *copyToNJBButton;
 	IBOutlet NSButton *copyFromNJBButton;
+	IBOutlet NSButton *deleteFromActiveObjectButton;
 	FileSystemBrowser *browser;
 	activeObjects activeObject;
 }
 - (IBAction)copyFromNJB:(id)sender;
 - (IBAction)copyToNJB:(id)sender;
+- (IBAction)deleteFromActiveObject:(id)sender;
 
 - (void)drawerWriteTag:(Track *)modifiedTrack;
 - (void)onFirstResponderChange:(NSResponder *)aResponder;
@@ -46,5 +48,7 @@ typedef enum _activeObjects {OBJECT_NJB_TABLE = 0, OBJECT_FILE_BROWSER = 1, OBJE
 - (NSString *)browserDirectory;
 - (void)applicationTerminating:(NSNotification *)note;
 - (NSString *)replaceBadFilenameChars:(NSString *)filename;
+- (BOOL)canDelete;
+- (void)addToFullItemArray:(MyItem *)item;
 
 @end
