@@ -9,6 +9,7 @@
 
 #import "MyFileTab.h"
 #import "defs.h"
+#import "MyNSWindow.h"
 
 @implementation MyFileTab
 
@@ -84,7 +85,7 @@
 
 /* can we delete something?
  */
-- (BOOL)menuShouldDelete
+- (BOOL)canDelete
 {
 	switch ([self activeObject])
 	{
@@ -104,7 +105,12 @@
 			break;
 		default:
 			return NO;
-	}	
+	}		
+}
+
+- (BOOL)menuShouldDelete
+{
+	return [self canDelete];
 }
 
 /* called by onFirstResponderChange: to tell us which object is
@@ -122,12 +128,16 @@
 
 - (void)onFirstResponderChange:(NSResponder *)aResponder
 {
+	// don't do anything if it's the window - we will have been called with another object
+	if ([aResponder class] == [MyNSWindow class])
+		return;
 	if (aResponder == njbTable)
 		[self setActiveObject:OBJECT_NJB_TABLE];
 	else if ([aResponder class] == [NSMatrix class])
 		[self setActiveObject:OBJECT_FILE_BROWSER];
 	else
 		[self setActiveObject:OBJECT_OTHER];
+	[deleteFromActiveObjectButton setEnabled:[self canDelete]];
 }
 
 /* can we copy a file to the NJB?
@@ -173,20 +183,16 @@
 		[self disableTransfers];
 		return;
 	}
-	if ([self canCopyToNJB])
-		[copyToNJBButton setEnabled:YES];
-	else
-		[copyToNJBButton setEnabled:NO];
-	if ([self canCopyFromNJB])
-		[copyFromNJBButton setEnabled:YES];
-	else
-		[copyFromNJBButton setEnabled:NO];
+	[deleteFromActiveObjectButton setEnabled:[self canDelete]];
+	[copyToNJBButton setEnabled:[self canCopyToNJB]];
+	[copyFromNJBButton setEnabled:[self canCopyFromNJB]];
 }
 
 /* disable transfer buttons
  */
 - (void)disableTransfers
 {
+	[deleteFromActiveObjectButton setEnabled:[self canDelete]];
 	[copyToNJBButton setEnabled:NO];
 	[copyFromNJBButton setEnabled:NO];
 }
@@ -320,4 +326,21 @@
 	return [browser directory];
 }
 
+/* user clicked on delete button
+ */
+- (IBAction)deleteFromActiveObject:(id)sender
+{
+	[self menuDelete];
+}
+
+/* adds the item to fullItemArray and calls
+* searchNJBTable. To be run when new item added to
+* list.
+*/
+- (void)addToFullItemArray:(MyItem *)item
+{
+	[fullItemArray addObject:item];
+	[self searchNJBTable];
+}
+
 @end
