@@ -5,6 +5,9 @@
 //  Created by Richard Low on Wed Jul 21 2004.
 //
 
+// todo: can we find out how many tracks there are before we download for progress bar & set capacity in NSMutableArray?
+// update libnjb to current cvs
+
 /* An obj-c wrapper for libnjb
  * most methods return an NJBTransactionResult
  * to indicate success/failure
@@ -148,6 +151,8 @@ StatusDisplayer *statusDisplayerGlobal;
 - (NSString *)ownerString
 {
 	char *ownerString = NJB_Get_Owner_String (njb);
+	if (ownerString == NULL)
+		return nil;
 	NSString *nsOwnerString = [[NSString alloc] initWithUTF8String:ownerString];
 	free(ownerString);
 	return [nsOwnerString autorelease];
@@ -176,8 +181,9 @@ StatusDisplayer *statusDisplayerGlobal;
 		str = songid_label(songtag, FR_GENRE);
 		if (str != NULL)
 			[track setGenre:[NSString stringWithUTF8String:str]];
+		// this is not used: we don't get extended track info from njb3
+		// njb1 gets it but ignored
 		str = songid_label(songtag, FR_FNAME);
-		// this appears to be always blank on mine
 		if (str != NULL)
 			[track setFilename:[NSString stringWithUTF8String:str]];
 		[track setFilesize:(unsigned)songid_size(songtag)];
@@ -196,14 +202,19 @@ StatusDisplayer *statusDisplayerGlobal;
 			[track setTrackNumber:(unsigned)songid_track(songtag)];
 		
 		str = songid_label(songtag, FR_CODEC);
-		if (strcmp(str, NJB_CODEC_MP3) == 0)
-			[track setCodec:CODEC_MP3];
-		else if (strcmp(str, NJB_CODEC_WMA) == 0)
-			[track setCodec:CODEC_WMA];
-		else if (strcmp(str, NJB_CODEC_WAV) == 0)
-			[track setCodec:CODEC_WAV];
-		else
+		if (str == NULL)
 			[track setCodec:CODEC_UNDEFINED];
+		else
+		{
+			if (strcmp(str, NJB_CODEC_MP3) == 0)
+				[track setCodec:CODEC_MP3];
+			else if (strcmp(str, NJB_CODEC_WMA) == 0)
+				[track setCodec:CODEC_WMA];
+			else if (strcmp(str, NJB_CODEC_WAV) == 0)
+				[track setCodec:CODEC_WAV];
+			else
+				[track setCodec:CODEC_UNDEFINED];
+		}
 		
 		[track setTrackID:songtag->trid];
 		
@@ -226,18 +237,24 @@ StatusDisplayer *statusDisplayerGlobal;
  */
 - (NJBTransactionResult *)uploadTrack:(Track *)track
 {
+	NSDate *date = [NSDate date];
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	NSLog(@"Uploading track: %@", [track fullPath]);
 	[self setStatus:STATUS_UPLOADING_TRACK];
 	int trackid;
-	if (NJB_Send_Track(njb, [[track fullPath] UTF8String], [[track njbCodec] UTF8String], [[track title] UTF8String], [[track album] UTF8String], [[track genre] UTF8String],
+	if (NJB_Send_Track(njb, [[track fullPath] fileSystemRepresentation], [[track njbCodec] UTF8String], [[track title] UTF8String], [[track album] UTF8String], [[track genre] UTF8String],
 											[[track artist] UTF8String], [track length], [track trackNumber], [[NSString stringWithFormat:@"%d", [track year]] UTF8String], 0, progress, NULL, &trackid) == -1 ) {
 		njb_error_dump(stderr);
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	} else {
 		[self updateDiskSpace];
 		[track setTrackID:trackid];
+		
+		double time = -[date timeIntervalSinceNow];
+		double rate = (double)[track filesize] / time;
+		NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
+		
 		return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 	}
 }
@@ -277,11 +294,16 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (NJBTransactionResult *)downloadTrack:(Track *)track
 {
-	if (NJB_Get_Track(njb, [track trackID], [track filesize], [[track fullPath] UTF8String], progress, NULL) == -1)
+	NSDate *date = [NSDate date];
+	if (NJB_Get_Track(njb, [track trackID], [track filesize], [[track fullPath] fileSystemRepresentation], progress, NULL) == -1)
 	{
 		njb_error_dump(stderr);
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	}
+	double time = -[date timeIntervalSinceNow];
+	double rate = (double)[track filesize] / time;
+	NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
+	
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
@@ -329,11 +351,16 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (NJBTransactionResult *)downloadFile:(DataFile *)dataFile
 {
-	if (NJB_Get_File(njb, [dataFile fileID], [dataFile size], [[dataFile fullPath] UTF8String], progress, NULL) == -1)
+	NSDate *date = [NSDate date];
+	if (NJB_Get_File(njb, [dataFile fileID], [dataFile size], [[dataFile fullPath] fileSystemRepresentation], progress, NULL) == -1)
 	{
 		njb_error_dump(stderr);
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	}
+	double time = -[date timeIntervalSinceNow];
+	double rate = (double)[dataFile size] / time;
+	NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
+		
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
@@ -342,14 +369,19 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	NSLog(@"Uploading file: %@", [dataFile fullPath]);
+	NSDate *date = [NSDate date];
 	[self setStatus:STATUS_UPLOADING_FILE];
 	int fileid;
-	if (NJB_Send_File(njb, [[dataFile fullPath] UTF8String], [[[dataFile fullPath] lastPathComponent] UTF8String], progress, NULL, &fileid) == -1 ) {
+	if (NJB_Send_File(njb, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] UTF8String], progress, NULL, &fileid) == -1 ) {
 		njb_error_dump(stderr);
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	} else {
 		[dataFile setFileID:fileid];
 		[self updateDiskSpace];
+		
+		double time = -[date timeIntervalSinceNow];
+		double rate = (double)[dataFile size] / time;
+		NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
 		return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 	}
 }
@@ -373,6 +405,8 @@ StatusDisplayer *statusDisplayerGlobal;
 - (NSCalendarDate *)jukeboxTime
 {
 	njb_time_t *time = NJB_Get_Time(njb);
+	if (time == NULL)
+		return nil;
 	
 	// assume njb time is our timezone
 	NSCalendarDate *date = [NSCalendarDate dateWithYear:time->year month:time->month day:time->day hour:time->hours minute:time->minutes second:time->seconds
