@@ -130,8 +130,9 @@ int NJB_Open (njb_t *njb)
 	      || njb->device_type == NJB_DEVICE_NJB3) {
 	    njb3_capture(njb);
 	  }
-		
+
 	  ret = NJB_Handshake(njb);
+
 	}
 
 	__leave;
@@ -155,7 +156,7 @@ void NJB_Close (njb_t *njb)
 	  if (njb->device_type == NJB_DEVICE_NJBZEN
 	      || njb->device_type == NJB_DEVICE_NJB3
 	      ) {
-			njb3_ping(njb, NULL, 1);
+	    njb3_ping(njb, NULL, 1);
 	  }
 	  njb3_release(njb);
 
@@ -246,7 +247,7 @@ int NJB_Handshake (njb_t *njb)
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
 
-		if ( njb3_ping(njb, &njbid, 0) == -1 ) {
+	  if ( njb3_ping(njb, &njbid, 0) == -1 ) {
 	    __leave;
 	    return -1;
 	  }
@@ -363,7 +364,17 @@ void NJB_Reset_Get_Track_Tag (njb_t *njb)
 
 	njb_error_clear();
 
-	njb->reset_get_track_tag = 1;
+	if (njb->device_type == NJB_DEVICE_NJB1) {
+	  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+	  state->reset_get_track_tag = 1;
+	}
+
+	if (PROTOCOL3_DEVICE(njb->device_type)) {
+	  if (njb3_reset_get_track_tag(njb) == -1) {
+	    /* FIXME: do something with this error some day. */
+	  }
+	}
 
 	__leave;
 }
@@ -380,10 +391,11 @@ songid_t *NJB_Get_Track_Tag (njb_t *njb)
 
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 		njbttaghdr_t tagh;
+		njb_state_t *state = (njb_state_t *) njb->protocol_state;
 
-		if ( njb->reset_get_track_tag ) {
+		if ( state->reset_get_track_tag ) {
 			status= njb_get_first_track_tag_header(njb, &tagh);
-			njb->reset_get_track_tag= 0;
+			state->reset_get_track_tag = 0;
 		} else {
 			status= njb_get_next_track_tag_header(njb, &tagh);
 		}
@@ -391,14 +403,7 @@ songid_t *NJB_Get_Track_Tag (njb_t *njb)
 	}
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  njb3_state_t *state = (njb3_state_t *) njb->protocol_state;
-
-	  if ( njb->reset_get_track_tag ) {
-	    ret=njb3_get_first_track_tag(njb, state->get_extended_tag_info);
-	    njb->reset_get_track_tag= 0;
-	  } else {
-	    ret=njb3_get_next_track_tag(njb);
-	  }
+	  ret = njb3_get_next_track_tag(njb);
 	}
 
 	__leave;
@@ -413,7 +418,17 @@ void NJB_Reset_Get_Playlist (njb_t *njb)
 
 	njb_error_clear();
 
-	njb->reset_get_playlist= 1;
+	if (njb->device_type == NJB_DEVICE_NJB1) {
+	  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+	  state->reset_get_playlist= 1;
+	}
+
+	if (PROTOCOL3_DEVICE(njb->device_type)) {
+	  if (njb3_reset_get_playlist_tag(njb) == -1) {
+	    /* FIXME: do something about this error eventually */
+	  }
+	}
 
 	__leave;
 }
@@ -430,8 +445,9 @@ playlist_t *NJB_Get_Playlist (njb_t *njb)
 	njb_error_clear();
 
 	if (njb->device_type == NJB_DEVICE_NJB1) {
+	        njb_state_t *state = (njb_state_t *) njb->protocol_state;
 
-		if ( njb->reset_get_playlist ) {
+		if ( state->reset_get_playlist ) {
 			while (retry) {
 				if ( njb_get_first_playlist_header(njb, &plh) == -1 ) {
 					if ( njb_error == EO_AGAIN && retry ) retry--;
@@ -441,7 +457,7 @@ playlist_t *NJB_Get_Playlist (njb_t *njb)
 					}
 				} else break;
 			}
-			njb->reset_get_playlist= 0;
+			state->reset_get_playlist= 0;
 		} else {
 			while (retry) {
 				if ( njb_get_next_playlist_header(njb, &plh) == -1 ) {
@@ -454,17 +470,11 @@ playlist_t *NJB_Get_Playlist (njb_t *njb)
 			}
 		}
 
-		ret= njb_get_playlist(njb, &plh);
+		ret = njb_get_playlist(njb, &plh);
 	}
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  if ( njb->reset_get_playlist ) {
-	    ret = njb3_get_first_playlist_tag (njb);
-	    njb->reset_get_playlist= 0;
-	  }
-	  else {
-	    ret = njb3_get_next_playlist_tag(njb);
-	  }
+	  ret = njb3_get_next_playlist_tag(njb);
 	}
 
 	__leave;
@@ -593,7 +603,17 @@ void NJB_Reset_Get_Datafile_Tag (njb_t *njb)
 
 	njb_error_clear();
 
-	njb->reset_get_datafile_tag= 1;
+	if (njb->device_type == NJB_DEVICE_NJB1 ) {
+	  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+	  state->reset_get_datafile_tag = 1;
+	}
+
+	if (PROTOCOL3_DEVICE(njb->device_type)) {
+	  if (njb3_reset_get_datafile_tag(njb) == -1) {
+	    /* FIXME: handle this error eventually */
+	  }
+	}
 
 	__leave;
 }
@@ -610,23 +630,20 @@ datafile_t *NJB_Get_Datafile_Tag (njb_t *njb)
 	njb_error_clear();
 
 	if (njb->device_type == NJB_DEVICE_NJB1 ) {
-		if ( njb->reset_get_datafile_tag ) {
-			status= njb_get_first_datafile_header(njb, &dfh);
-			njb->reset_get_datafile_tag= 0;
+	        njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+		if ( state->reset_get_datafile_tag ) {
+			status = njb_get_first_datafile_header(njb, &dfh);
+			state->reset_get_datafile_tag= 0;
 		} else {
-			status= njb_get_next_datafile_header(njb, &dfh);
+			status = njb_get_next_datafile_header(njb, &dfh);
 		}
 
 		ret= ( status == -1 ) ? NULL : njb_get_datafile_tag(njb, &dfh);
 	}
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  if ( njb->reset_get_datafile_tag ) {
-	    ret= njb3_get_first_datafile_tag(njb);
-	    njb->reset_get_datafile_tag= 0;
-	  } else {
-	    ret= njb3_get_next_datafile_tag(njb);
-	  }
+	  ret = njb3_get_next_datafile_tag(njb);
 	}
 
 	__leave;
@@ -1431,10 +1448,11 @@ int NJB_Send_File (njb_t *njb, const char *path, const char *name,
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 
 		if ( (phdr = datafile_pack(&df, &fhsize)) == NULL ) return -1;
-		fh.size= fhsize;
+		fh.size = fhsize;
 
 		if ( njb_send_datafile_tag(njb, &fh, phdr) == -1 ) {
-			if ( njb_error == EO_BADSTATUS ) NJB_ERROR(EO_XFERDENIED);
+			if ( njb_error == EO_BADSTATUS ) 
+			  NJB_ERROR(EO_XFERDENIED);
 			free(phdr);
 			__leave;
 			return -1;
